<?xml version="1.0" encoding="UTF-8"?>
<xsl:stylesheet version="1.0" xmlns:xsl="http://www.w3.org/1999/XSL/Transform">
  <xsl:output method="html"/>
  <xsl:template match="/">
    <html>
      <head>
        <title>
          <xsl:for-each select="object">
            <xsl:value-of select="@name"/>
          </xsl:for-each>
        </title>
        <link rel="stylesheet" type="text/css" media="all"
              href="../xsl.css" />      
      </head>
      <body>
        <xsl:for-each select="object">
          <h2>
            <xsl:value-of select="@name"/>
          </h2> 
        </xsl:for-each>
        <h1>Properties</h1>
        <table>
          <xsl:for-each select="object/property">
            <xsl:variable name="color">
              <xsl:choose>
                <xsl:when test="position() mod 2 = 0">
                  <xsl:text>#ffffff</xsl:text>
                </xsl:when>
                <xsl:otherwise>
                  <xsl:text>#eeeeee</xsl:text>
                </xsl:otherwise>
              </xsl:choose>
            </xsl:variable>
            <tr style="background-color:{$color}">
              <td colspan="">
                <xsl:value-of select="@name"/>
              </td>
              <td colspan="">
                <xsl:value-of select="text()"/>
              </td>
              <td>
                <xsl:for-each select="element">
                  <xsl:value-of select="@value"/><br />
                </xsl:for-each>
              </td>
            </tr>
          </xsl:for-each>
        </table>
        <h1>Alias</h1>
        <table>
          <th>From</th><th>To</th>
          <xsl:for-each select="object/alias">
            <xsl:variable name="color">
              <xsl:choose>
                <xsl:when test="position() mod 2 = 0">
                  <xsl:text>#ffffff</xsl:text>
                </xsl:when>
                <xsl:otherwise>
                  <xsl:text>#eeeeee</xsl:text>
                </xsl:otherwise>
              </xsl:choose>
            </xsl:variable>
            <tr style="background-color:{$color}">
              <td colspan="">
                <xsl:value-of select="@from"/>
              </td>
              <td colspan="">
                <xsl:value-of select="@to"/>
              </td>
            </tr>
          </xsl:for-each>
        </table>
        <h1>Actions</h1>
        <table>
          <tr>
            <th>Id</th>
            <th>Title</th>
            <th>Permission</th>
            <th>Category</th>
            <th>Visible</th>
          </tr>
          <xsl:for-each select="object/action">
            <xsl:sort select="@action_id"
                      order="ascending" />
            <xsl:variable name="color">
              <xsl:choose>
                <xsl:when test="position() mod 2 = 0">
                  <xsl:text>#ffffff</xsl:text>
                </xsl:when>
                <xsl:otherwise>
                  <xsl:text>#eeeeee</xsl:text>
                </xsl:otherwise>
              </xsl:choose>
            </xsl:variable>
            <tr style="background-color:{$color}">
              <td>
                <xsl:value-of select="@action_id"/>
              </td>
              <td>
                <xsl:value-of select="@title"/>
              </td>
              <td>
                <xsl:for-each select="permission">
                  <xsl:value-of select="@value"/>
                </xsl:for-each>
              </td>
              <td>
                <xsl:value-of select="@category"/>
              </td>
              <td>
                <xsl:value-of select="@visible"/>
              </td>
              <xsl:for-each select="element">
                <td>
                  <xsl:value-of select="@value"/>
                </td>
              </xsl:for-each>
            </tr>
            <tr style="background-color:{$color}">
              <th>url_expression</th>
              <td colspan="4">
                <xsl:value-of select="@url_expr"/>
              </td>
            </tr>
            <tr style="background-color:{$color}">
              <th>condition_expression</th>
              <td colspan="4">
                <xsl:value-of select="@condition_expr"/>
              </td>
            </tr>
          </xsl:for-each>
        </table>
      </body>
    </html>
  </xsl:template>
</xsl:stylesheet>
